import pyglet 
import ratcave as rc
from pyglet.window import key
from pyglet.window import mouse
import math

# new controller stuff
joysticks = pyglet.input.get_joysticks()
assert joysticks, 'No joystick device is connected'
joystick = joysticks[0]
joystick.open()

# Create Window and handle inputs
window = pyglet.window.Window()
keys = key.KeyStateHandler()


#update window
def update(dt):
    pass
pyglet.clock.schedule(update)

#CREATE OBJECT
# Insert filename into WavefrontReader.
obj_filename = rc.resources.obj_primitives
obj_reader = rc.WavefrontReader(obj_filename)
# Create Mesh
monkey = obj_reader.get_mesh("Monkey", scale=0.3)
monkey.position.xyz = 0, 0, 0
#create colors
monkey.uniforms['diffuse']= [0.7,0.2,0.2]
monkey.uniforms['ambient']=[0.5,0.5,0.5]


# Create Scene
scene = rc.Scene(meshes=[monkey])
scene.bgColor = 1,0,0.5
scene.light = rc.light.Light(position=(-5,10,0))
shader = rc.Shader.from_file(*rc.resources.genShader)

#global variables for different things

global w 				#for phi for paning camera
w=0.1

global x1 				#variables for creating vector
x1 = 0
global x2
x2 = 0
global y1
y1 = 0
global y2
y2 = 0
global grav                             #variabe for creating grav
grav = 0


#define functions for event handling
@window.event
def on_draw():
    with shader:
        scene.draw()


#HANDLE MOUSE INPUT
def on_mouse_press(x,y,buttons,modifiers):
		global x1, y1
		x1 = x
		y1 = y
def on_mouse_release(x, y, buttons, modifiers):
		global x1, y1, x2, y2
		x2 = x
		y2 = y
		#create vektor
		v_x = x2 - x1 
		v_y = y2 - y1
		print(v_x, v_y)
		return v_x, v_y


def move_camera(dt):
    camera_speed = 3
    if keys[key.LEFT]:
        scene.camera.position.x += camera_speed * dt
    if keys[key.RIGHT]:
        scene.camera.position.x -= camera_speed * dt
    if keys[key.UP]:
        scene.camera.position.y -= camera_speed * dt
    if keys[key.DOWN]:
        scene.camera.position.y += camera_speed * dt
    if keys[key.MINUS]:
        scene.camera.position.z *= (1+dt)
    if keys[key.PLUS]:
        scene.camera.position.z /= (1+dt)
    if keys[key.O]:
        global w
        w=w+0.05
        scene.camera.position.x = 4*math.sin(w)
        scene.camera.position.z = 4*math.cos(w)
        scene.camera.rotation.y += 2.87 
        print(scene.camera.position.xyz)
    if keys[key.P]:
        w=w-0.05
        scene.camera.position.x = 4*math.sin(w)
        scene.camera.position.z = 4*math.cos(w)
        scene.camera.rotation.y -= 2.87
        #print(scene.camera.rotation.xyz)

def move_sphere(dt):
    if keys[key.A]:
        monkey.position.x -=0.05
    if keys[key.D]:
        monkey.position.x +=0.05
    if keys[key.W]:
        monkey.position.y +=0.05
    if keys[key.S]:
        monkey.position.y -=0.05
    if keys[key.Q]:
        monkey.rotation.y += 5000 
    if keys[key.E]:
        monkey.rotation.y -= 5000 

def gravity(dt):

    if monkey.position.y >=0:
        if not keys[key.W]:
            global grav
            monkey.position.y = monkey.position.y - grav
            grav += 0.004
    if monkey.position.y <= 0:
        grav = 0

def y_tracker(dt):
    print(monkey.position.y)

#controller events
@joystick.event
def on_joyaxis_motion(joystick, axis, button):
    if button >= 0.1:
        print(joystick, axis, button)
    if button <= -0.1:
        print(joystick, axis, button)
    a = 0.01*button
    if axis == 'x':
        monkey.position.x += a
    if axis == 'y':
        monkey.position.y += a
    if axis == 'z':
        scene.camera.position.z += a*10
    if axis == 'rx':
        if button >= 0.1 or button <= -0.1:
            scene.camera.rotation.y += button
    if axis == 'ry':
        if button >= 0.1 or button <= -0.1:
            scene.camera.rotation.x += button

@joystick.event
def  on_joyhat_motion(joystick, hat_x, hat_y):
    if hat_x == -1:
        scene.camera.position.x -= 0.05
    if hat_x == 1:
        scene.camera.position.x += 0.05
    if hat_y == -1:
        scene.camera.position.y -= 0.05
    if hat_y == 1:
        scene.camera.position.y += 0.05

@joystick.event
def on_joybutton_press(joystick, button):
    print(button)
    if button == 0:
        monkey.position.y +=5
    if button == 4:
        monkey.rotation.y +=10
    if button == 5:
        monkey.rotation.y -=10

@joystick.event
def jump(joystick, button):
    if button == 0:
        monkey.position.y +=3

window.push_handlers(keys)
window.push_handlers(on_mouse_press)
window.push_handlers(on_mouse_release)

pyglet.clock.schedule(y_tracker)
pyglet.clock.schedule(gravity)
pyglet.clock.schedule(move_camera)
pyglet.clock.schedule(move_sphere)
pyglet.clock.schedule_interval(update, 1/60.0)

#camera position
scene.camera.position.z = 4

pyglet.app.run()
